@@ -335,10 +335,19 @@ int fdisk_apply_user_device_properties(struct fdisk_context *cxt)
 
 	if (cxt->user_pyh_sector)
 		cxt->phy_sector_size = cxt->user_pyh_sector;
-	if (cxt->user_log_sector)
+	if (cxt->user_log_sector) {
+		uint64_t old_total = cxt->total_sectors;
+		uint64_t old_secsz = cxt->sector_size;
+
 		cxt->sector_size = cxt->min_io_size =
 			cxt->io_size = cxt->user_log_sector;
 
+		if (cxt->sector_size != old_secsz) {
+			cxt->total_sectors = (old_total * (old_secsz/512)) / (cxt->sector_size >> 9);
+			DBG(CXT, ul_debugobj(cxt, "new total sectors: %ju", cxt->total_sectors));
+		}
+	}
+
 	if (cxt->user_geom.heads)
 		cxt->geom.heads = cxt->user_geom.heads;
 	if (cxt->user_geom.sectors)
@@ -657,6 +666,7 @@ int fdisk_reread_partition_table(struct fdisk_context *cxt)
 
 	i = fstat(cxt->dev_fd, &statbuf);
 	if (i == 0 && S_ISBLK(statbuf.st_mode)) {
+		DBG(CXT, ul_debugobj(cxt, "calling re-read ioctl"));
 		sync();
 #ifdef BLKRRPART
 		fdisk_info(cxt, _("Calling ioctl() to re-read partition table."));
