@@ -121,7 +121,7 @@ static int has_discard(const char *devname, struct sysfs_cxt *wholedisk)
 		return 1;
 	/*
 	 * This is tricky to read the info from sys/, because the queue
-	 * atrributes are provided for whole devices (disk) only. We're trying
+	 * attributes are provided for whole devices (disk) only. We're trying
 	 * to reuse the whole-disk sysfs context to optimize this stuff (as
 	 * system usually have just one disk only).
 	 */
@@ -158,23 +158,11 @@ static int uniq_fs_source_cmp(
 		struct libmnt_fs *a,
 		struct libmnt_fs *b)
 {
-	int eq;
-
 	if (mnt_fs_is_pseudofs(a) || mnt_fs_is_netfs(a) ||
 	    mnt_fs_is_pseudofs(b) || mnt_fs_is_netfs(b))
 		return 1;
 
-	eq = mnt_fs_streq_srcpath(a, mnt_fs_get_srcpath(b));
-	if (eq) {
-		const char *aroot = mnt_fs_get_root(a),
-			   *broot = mnt_fs_get_root(b);
-		if (!aroot || !broot)
-			eq = 0;
-		else if (strcmp(aroot, broot) != 0)
-			eq = 0;
-	}
-
-	return !eq;
+	return !mnt_fs_streq_srcpath(a, mnt_fs_get_srcpath(b));
 }
 
 /*
@@ -220,7 +208,7 @@ static int fstrim_all(struct fstrim_range *rangetpl, int verbose)
 			continue;
 
 		/* Is it really accessible mountpoint? Not all mountpoints are
-		 * accessible (maybe over mounted by another fylesystem) */
+		 * accessible (maybe over mounted by another filesystem) */
 		path = mnt_get_mountpoint(tgt);
 		if (path && strcmp(path, tgt) == 0)
 			rc = 0;
