@@ -86,6 +86,7 @@
 #define _PATH_PROC_BC	"/proc/bc"
 #define _PATH_PROC_DEVICETREE	"/proc/device-tree"
 #define _PATH_DEV_MEM 		"/dev/mem"
+#define _PATH_PROC_OSRELEASE	"/proc/sys/kernel/osrelease"
 
 /* Xen Domain feature flag used for /sys/hypervisor/properties/features */
 #define XENFEAT_supervisor_mode_kernel		3
@@ -125,7 +126,8 @@ const char *hv_vendors[] = {
 	[HYPER_VBOX]	= "Oracle",
 	[HYPER_OS400]	= "OS/400",
 	[HYPER_PHYP]	= "pHyp",
-	[HYPER_SPAR]	= "Unisys s-Par"
+	[HYPER_SPAR]	= "Unisys s-Par",
+	[HYPER_WSL]	= "Windows Subsystem for Linux"
 };
 
 const int hv_vendor_pci[] = {
@@ -654,13 +656,10 @@ read_basicinfo(struct lscpu_desc *desc, struct lscpu_modifier *mod)
 	if (mod->system == SYSTEM_LIVE)
 		read_physical_info_powerpc(desc);
 
-	if (path_exist(_PATH_PROC_SYSINFO)) {
-		FILE *fd = path_fopen("r", 0, _PATH_PROC_SYSINFO);
-
-		while (fd && fgets(buf, sizeof(buf), fd) != NULL && !desc->machinetype)
+	if ((fp = path_fopen("r", 0, _PATH_PROC_SYSINFO))) {
+		while (fgets(buf, sizeof(buf), fp) != NULL && !desc->machinetype)
 			lookup(buf, "Type", &desc->machinetype);
-		if (fd)
-			fclose(fd);
+		fclose(fp);
 	}
 }
 
@@ -914,6 +913,22 @@ read_hypervisor(struct lscpu_desc *desc, struct lscpu_modifier *mod)
 {
 	FILE *fd;
 
+	/* We have to detect WSL first. is_vmware_platform() crashes on Windows 10. */
+
+	if ((fd = path_fopen("r", 0, _PATH_PROC_OSRELEASE))) {
+		char buf[256];
+
+		if (fgets(buf, sizeof(buf), fd) != NULL) {
+			if (strstr(buf, "Microsoft")) {
+				desc->hyper = HYPER_WSL;
+				desc->virtype = VIRT_CONT;
+			}
+		}
+		fclose(fd);
+		if (desc->virtype)
+			return;
+	}
+
 	if (mod->system != SYSTEM_SNAPSHOT) {
 		read_hypervisor_cpuid(desc);
 		if (!desc->hyper)
@@ -973,16 +988,13 @@ read_hypervisor(struct lscpu_desc *desc, struct lscpu_modifier *mod)
 		desc->virtype = VIRT_FULL;
 
 	/* IBM PR/SM */
-	} else if (path_exist(_PATH_PROC_SYSINFO)) {
-		FILE *sysinfo_fd = path_fopen("r", 0, _PATH_PROC_SYSINFO);
+	} else if ((fd = path_fopen("r", 0, _PATH_PROC_SYSINFO))) {
 		char buf[BUFSIZ];
 
-		if (!sysinfo_fd)
-			return;
 		desc->hyper = HYPER_IBM;
 		desc->hypervisor = "PR/SM";
 		desc->virtype = VIRT_FULL;
-		while (fgets(buf, sizeof(buf), sysinfo_fd) != NULL) {
+		while (fgets(buf, sizeof(buf), fd) != NULL) {
 			char *str;
 
 			if (!strstr(buf, "Control Program:"))
@@ -1006,7 +1018,7 @@ read_hypervisor(struct lscpu_desc *desc, struct lscpu_modifier *mod)
 			while ((str = strstr(desc->hypervisor, "  ")))
 				memmove(str, str + 1, strlen(str));
 		}
-		fclose(sysinfo_fd);
+		fclose(fd);
 	}
 
 	/* OpenVZ/Virtuozzo - /proc/vz dir should exist
@@ -1028,11 +1040,10 @@ read_hypervisor(struct lscpu_desc *desc, struct lscpu_modifier *mod)
 		desc->virtype = VIRT_PARA;
 
 	/* Linux-VServer */
-	} else if (path_exist(_PATH_PROC_STATUS)) {
+	} else if ((fd = path_fopen("r", 0, _PATH_PROC_STATUS))) {
 		char buf[BUFSIZ];
 		char *val = NULL;
 
-		fd = path_fopen("r", 1, _PATH_PROC_STATUS);
 		while (fgets(buf, sizeof(buf), fd) != NULL) {
 			if (lookup(buf, "VxID", &val))
 				break;
@@ -1820,6 +1831,7 @@ print_summary(struct lscpu_desc *desc, struct lscpu_modifier *mod)
 	if (desc->nsockets) {
 		int threads_per_core, cores_per_socket, sockets_per_book;
 		int books_per_drawer, drawers;
+		FILE *fd;
 
 		threads_per_core = cores_per_socket = sockets_per_book = 0;
 		books_per_drawer = drawers = 0;
@@ -1831,8 +1843,7 @@ print_summary(struct lscpu_desc *desc, struct lscpu_modifier *mod)
 		 * If the cpu topology is not exported (e.g. 2nd level guest)
 		 * fall back to old calculation scheme.
 		 */
-		if (path_exist(_PATH_PROC_SYSINFO)) {
-			FILE *fd = path_fopen("r", 0, _PATH_PROC_SYSINFO);
+		if ((fd = path_fopen("r", 0, _PATH_PROC_SYSINFO))) {
 			char pbuf[BUFSIZ];
 			int t0, t1;
 
