@@ -100,7 +100,8 @@ enum {
 	LSNS_ID_PID,
 	LSNS_ID_UTS,
 	LSNS_ID_IPC,
-	LSNS_ID_USER
+	LSNS_ID_USER,
+	LSNS_ID_CGROUP
 };
 
 static char *ns_names[] = {
@@ -109,7 +110,8 @@ static char *ns_names[] = {
 	[LSNS_ID_PID] = "pid",
 	[LSNS_ID_UTS] = "uts",
 	[LSNS_ID_IPC] = "ipc",
-	[LSNS_ID_USER] = "user"
+	[LSNS_ID_USER] = "user",
+	[LSNS_ID_CGROUP] = "cgroup"
 };
 
 struct lsns_namespace {
@@ -215,6 +217,30 @@ static int get_ns_ino(int dir, const char *nsname, ino_t *ino)
 	return 0;
 }
 
+static int parse_proc_stat(FILE *fp, pid_t *pid, char *state, pid_t *ppid)
+{
+	char *line = NULL, *p;
+	size_t len = 0;
+	int rc;
+
+	if (getline(&line, &len, fp) < 0) {
+		rc = -errno;
+		goto error;
+	}
+
+	p = strrchr(line, ')');
+	if (p == NULL ||
+	    sscanf(line, "%d (", pid) != 1 ||
+	    sscanf(p, ") %c %d*[^\n]", state, ppid) != 2) {
+		rc = -EINVAL;
+		goto error;
+	}
+	rc = 0;
+
+error:
+	free(line);
+	return rc;
+}
 
 static int read_process(struct lsns *ls, pid_t pid)
 {
@@ -253,11 +279,9 @@ static int read_process(struct lsns *ls, pid_t pid)
 		rc = -errno;
 		goto done;
 	}
-	rc = fscanf(f, "%d %*s %c %d*[^\n]", &p->pid, &p->state, &p->ppid);
-	if (rc != 3) {
-		rc = rc < 0 ? -errno : -EINVAL;
+	rc = parse_proc_stat(f, &p->pid, &p->state, &p->ppid);
+	if (rc < 0)
 		goto done;
-	}
 	rc = 0;
 
 	for (i = 0; i < ARRAY_SIZE(p->ns_ids); i++) {
@@ -470,8 +494,8 @@ static void add_scols_line(struct lsns *ls, struct libscols_table *table,
 			break;
 		}
 
-		if (str)
-			scols_line_set_data(line, i, str);
+		if (str && scols_line_refer_data(line, i, str) != 0)
+			err_oom();
 	}
 
 	proc->outline = line;
@@ -599,7 +623,7 @@ static void __attribute__ ((__noreturn__)) usage(FILE * out)
 	fputs(_(" -p, --task <pid>       print process namespaces\n"), out);
 	fputs(_(" -r, --raw              use the raw output format\n"), out);
 	fputs(_(" -u, --notruncate       don't truncate text in columns\n"), out);
-	fputs(_(" -t, --type <name>      namespace type (mnt, net, ipc, user, pid, uts)\n"), out);
+	fputs(_(" -t, --type <name>      namespace type (mnt, net, ipc, user, pid, uts, cgroup)\n"), out);
 
 	fputs(USAGE_SEPARATOR, out);
 	fputs(USAGE_HELP, out);
